#!/usr/bin/env python
# coding: utf-8

# In[1]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go
import pycountry_convert as pc
from sklearn.linear_model import LinearRegression


# In[2]:


# load data
fish_hg_local = pd.read_excel('Fish-MeHg-Country.xlsx')                                   # mg/kg
fish_hg_global = pd.read_excel('Fish-MeHg-Global.xlsx')                                   # mg/kg
fish_cons = pd.read_excel('../Food Consumption/Fish-Consumption-Country.xlsx')            # kg/capital/year
lavoie_data = pd.read_excel('../StonyBrook Database/Lavoie_data.xlsx')                    # ug of MeHg/kg of body mass (BM) / week
rice_hg = pd.read_excel('mercury in rice.xlsx','country')                                 # rice MeHg concentration, ng/g
rice_cons = pd.read_excel('../Food Consumption/Rice-Consumption-Country.xlsx')            # rice consumption, kg/capital/year


# In[3]:


# only select 2011 data
lavoie_data = lavoie_data[['alpha3',2011]]


# In[4]:


# functions to get values
def get_fish_hg(fish_hg,alpha3,water,farm,trophic):
    select_fish_hg = fish_hg[(fish_hg['alpha3']==alpha3) & (fish_hg['water']==water) &     (fish_hg['farm']==farm) & (fish_hg['trophic']==trophic)]
    if select_fish_hg.size > 0:
        return [select_fish_hg['Hg'].values[0],select_fish_hg['std'].values[0]]
    else:
        return [np.nan,np.nan]

def get_fish_cons(fish_cons,alpha3,type):
    select_fish_cons = fish_cons[fish_cons['alpha3']==alpha3]
    if select_fish_cons.size > 0:
        return select_fish_cons[type].values[0]
    else:
        return np.nan

def get_rice_hg(rice_hg,alpha3):
    select_rice_hg = rice_hg[rice_hg['alpha3']==alpha3]
    if select_rice_hg.size > 0:
        return [select_rice_hg['MeHg (ng/g)'].values[0],select_rice_hg['stdev'].values[0]]
    else:
        return [np.nan,np.nan]

def get_rice_cons(rice_cons,alpha3):
    select_rice_cons = rice_cons[rice_cons['alpha3']==alpha3]
    if select_rice_cons.size > 0:
        return select_rice_cons['Rice (Milled Equivalent, kg)'].values[0]
    else:
        return np.nan


# In[5]:


# get continent from alpha3
def get_continent(alpha3):
    if alpha3 == 'BYS': # These countries cannot be handled by the pycountry_convert library
        continent = 'EU'
    elif alpha3 == 'ZAR':
        continent = 'AF'
    elif alpha3 == 'ATF':
        continent = 'AF'
    elif alpha3 == 'GBM':
        continent = 'EU'
    elif alpha3 == 'JTN':
        continent = 'NA'
    elif alpha3 == 'UNK':
        continent = 'EU'
    elif alpha3 == 'MID':
        continent = 'NA'   
    elif alpha3 == 'ANT':
        continent = 'SA'   
    elif alpha3 == 'MEX':  # Mexico belongs to NA geographically, but bw same as SA
        continent = 'SA'   
    elif alpha3 == 'PCN':
        continent = 'AS'     
    elif alpha3 == 'ESH':
        continent = 'AF' 
    elif alpha3 == 'WAK':
        continent = 'NA' 
    else:
        continent = pc.country_alpha2_to_continent_code(pc.country_alpha3_to_country_alpha2(alpha3))
    return continent


# In[6]:


# get body weight based on continent
def get_bw(continent):
    if continent == 'AS':
        bw = 57.7
    elif continent == 'NA':
        bw = 80.7
    elif continent == 'EU':
        bw = 70.8
    elif continent == 'SA':
        bw = 67.9
    elif continent == 'OC':
        bw = 74.1
    elif continent == 'AF':
        bw = 60.7
    else:
        bw = np.nan
    return bw


# In[8]:


# calculate exposure
countries_database_local = pd.DataFrame(columns=['alpha3','Freshwater Fish MeHg','Seawater Fish MeHg','Lavoie Seawater Fish MeHg','Rice MeHg'])
for alpha3 in fish_cons['alpha3']:
    if alpha3 == 'ATA' or pd.isna(alpha3):
        continue
        
    exposure1 = 0    
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_2') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Farmed','(1.99, 2.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_3') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Farmed','(2.49, 3.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_4') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Farmed','(3.49, 4.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_5') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Farmed','(4.49, 4.99]')[0]

    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_2') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Wild','(1.99, 2.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_3') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Wild','(2.49, 3.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_4') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Wild','(3.49, 4.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_5') * get_fish_hg(fish_hg_local,alpha3,'Freshwater','Wild','(4.49, 4.99]')[0]
        
    exposure2 = 0
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_2') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Farmed','(1.99, 2.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_3') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Farmed','(2.49, 3.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_4') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Farmed','(3.49, 4.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_5') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Farmed','(4.49, 4.99]')[0]

    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_2') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Wild','(1.99, 2.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_3') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Wild','(2.49, 3.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_4') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Wild','(3.49, 4.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_5') * get_fish_hg(fish_hg_local,alpha3,'Seawater','Wild','(4.49, 4.99]')[0]
    
    # look up Lavoie database
    exposure3 = np.nan
    for index,lavoie in lavoie_data.iterrows():
        if alpha3 == lavoie['alpha3']:
            exposure3 = lavoie[2011]
            break

    # calcualte rice MeHg exposure
    exposure4 = get_rice_cons(rice_cons,alpha3) * get_rice_hg(rice_hg,alpha3)[0]
    
    # Get body weight based on continent
    continent = get_continent(alpha3)
    bw = get_bw(continent)
    
    # convert exposure to ug MeHg per kg body weight per year
    exposure1 *= 1000.0 / bw
    exposure2 *= 1000.0 / bw
    exposure3 *= 52.1785
    exposure4 *= 1.0 / bw
    
    countries_database_local.loc[len(countries_database_local.index)+1] = [alpha3,exposure1,exposure2,exposure3,exposure4]


# In[9]:


# calculate exposure
countries_database_global = pd.DataFrame(columns=['alpha3','Freshwater Fish MeHg','Seawater Fish MeHg','Lavoie Seawater Fish MeHg','Rice MeHg'])
for alpha3 in fish_cons['alpha3']:
    if alpha3 == 'ATA' or pd.isna(alpha3):
        continue
        
    exposure1 = 0    
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_2') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Farmed','(1.99, 2.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_3') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Farmed','(2.49, 3.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_4') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Farmed','(3.49, 4.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Farm_5') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Farmed','(4.49, 4.99]')[0]

    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_2') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Wild','(1.99, 2.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_3') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Wild','(2.49, 3.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_4') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Wild','(3.49, 4.49]')[0]
    exposure1 += get_fish_cons(fish_cons,alpha3,'Fresh_Wild_5') * get_fish_hg(fish_hg_global,alpha3,'Freshwater','Wild','(4.49, 4.99]')[0]
        
    exposure2 = 0
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_2') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Farmed','(1.99, 2.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_3') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Farmed','(2.49, 3.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_4') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Farmed','(3.49, 4.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Farm_5') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Farmed','(4.49, 4.99]')[0]

    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_2') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Wild','(1.99, 2.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_3') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Wild','(2.49, 3.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_4') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Wild','(3.49, 4.49]')[0]
    exposure2 += get_fish_cons(fish_cons,alpha3,'Sea_Wild_5') * get_fish_hg(fish_hg_global,alpha3,'Seawater','Wild','(4.49, 4.99]')[0]
    
    # look up Lavoie database
    exposure3 = np.nan
    for index,lavoie in lavoie_data.iterrows():
        if alpha3 == lavoie['alpha3']:
            exposure3 = lavoie[2011]
            break

    # calcualte rice MeHg exposure
    exposure4 = get_rice_cons(rice_cons,alpha3) * get_rice_hg(rice_hg,alpha3)[0]
    
    # Get body weight based on continent
    continent = get_continent(alpha3)
    bw = get_bw(continent)
    
    # convert exposure to ug MeHg per kg body weight per year
    exposure1 *= 1000.0 / bw
    exposure2 *= 1000.0 / bw
    exposure3 *= 52.1785
    exposure4 *= 1.0 / bw

    countries_database_global.loc[len(countries_database_global.index)+1] = [alpha3,exposure1,exposure2,exposure3,exposure4]


# In[10]:


# a linear regression
temp = countries_database_local[['Seawater Fish MeHg','Lavoie Seawater Fish MeHg']].dropna(how="any")
X = temp['Seawater Fish MeHg'].to_numpy().reshape(-1,1)
y = temp['Lavoie Seawater Fish MeHg'].to_numpy().reshape(-1,1)
#reg = LinearRegression().fit(np.log10(X), np.log10(y))


# In[11]:


# plot vs Lavoie
fig, ax = plt.subplots(1,1, sharey=False,clear=True,figsize=(6,6))
ax.plot(np.log10(X),np.log10(y), 'b.')
ax.set_xlabel('Seawater Fish MeHg')
ax.set_ylabel('Lavoie Seawater Fish MeHg')
ax.grid(True)
ax.set_title('Local vs Lavoie')


# In[12]:


# a linear regression
temp = countries_database_global[['Seawater Fish MeHg','Lavoie Seawater Fish MeHg']].dropna(how="any")
X = temp['Seawater Fish MeHg'].to_numpy().reshape(-1,1)
y = temp['Lavoie Seawater Fish MeHg'].to_numpy().reshape(-1,1)
#reg = LinearRegression().fit(np.log10(X), np.log10(y))


# In[13]:


# plot vs Lavoie
fig, ax = plt.subplots(1,1, sharey=False,clear=True,figsize=(6,6))
ax.plot(np.log10(X),np.log10(y), 'b.')
ax.set_xlabel('Seawater Fish MeHg')
ax.set_ylabel('Lavoie Seawater Fish MeHg')
ax.grid(True)
ax.set_title('Global vs Lavoie')


# In[14]:


countries_database_local.to_excel('Exposure-local-MeHg-Country-ugkg-1year-1.xlsx')
countries_database_global.to_excel('Exposure-global-MeHg-Country-ugkg-1year-1.xlsx')
