#!/usr/bin/env python
# coding: utf-8

# In[15]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go


# In[16]:


# load excel data as dataframe
data=pd.read_excel('Fish-Hg-3D-data4.xlsx')


# In[17]:


# Exclude abnormal data
data=data[data['Exclude']!=1]


# In[18]:


# group based on TL
tl_bucket = [1.99,2.49,3.49,4.49,4.99]
data['Trophic Category'] = pd.cut(data['Trophic Level'],tl_bucket)


# In[19]:


# group by 3 categories
data_tl = data.groupby('Trophic Category')
data_3d = data.groupby(['Trophic Category','Freshwater/Seawater','Farmed/Wild'])


# In[20]:


# calculate the weighted mean
# function to calculate geomean, assuming normal or lognormal distribution
wm1 = lambda x: np.average(x,weights=np.sign(data.loc[x.index, "Sample Size"]))
wm2 = lambda x: np.std(x)
wm3 = lambda x: np.average(np.log10(x),weights=np.sign(data.loc[x.index, "Sample Size"]))
wm4 = lambda x: np.std(np.log10(x))
# weidhted mean, exclude outliers
def wm5(x):
    logx = np.log10(x).to_numpy()
    mean_logx = np.mean(logx)
    std_logx = np.std(logx)
    nx = np.sign(data.loc[x.index, "Sample Size"]).to_numpy()
    for i,logx_i in enumerate(logx):
        if (logx_i - mean_logx)/std_logx > 1:
            nx[i] = 0
    return np.average(logx,weights=nx)
def wm6(x):
    logx = np.log10(x).to_numpy()
    mean_logx = np.mean(logx)
    std_logx = np.std(logx)
    nx = np.sign(data.loc[x.index, "Sample Size"]).to_numpy()
    for i,logx_i in enumerate(logx):
        if (logx_i - mean_logx)/std_logx > 1:
            nx[i] = 0
    return np.sqrt(np.average((logx-mean_logx)**2, weights=nx))

Means_Hg = data_3d.agg(mean=("Hg Level", wm1))['mean']
GMeans_Hg = data_3d.agg(gmean=("Hg Level", wm3))['gmean']
Stds_Hg = data_3d.agg(std=("Hg Level", wm2))['std']
GStds_Hg = data_3d.agg(gstd=("Hg Level", wm4))['gstd']
GEMeans_Hg = data_3d.agg(gemean=("Hg Level", wm5))['gemean']
GEStds_Hg = data_3d.agg(gestd=("Hg Level", wm6))['gestd']


# In[21]:


# get country list
country_name=pd.read_excel('Shape_Country_Name.xlsx')


# In[22]:


# sample sized weighted mean and std， not using sample size as weights
def weighted_avg_and_std(values, weights):
    mean = np.average(values,weights=np.sqrt(weights))
    gmean = np.average(np.log10(values),weights=np.sqrt(weights))
    var = np.average((values-mean)**2, weights=np.sqrt(weights))
    gvar = np.average((np.log10(values)-gmean)**2, weights=np.sqrt(weights))
    return (mean, gmean, math.sqrt(var), math.sqrt(gvar))


# In[23]:


# put data into a database# calculate the weighted mean
countries_database = pd.DataFrame(columns=['country','alpha3','water','farm','trophic','Hg','std','flag'])   
for country in country_name.iloc:
    for Water in ['Seawater','Freshwater']:
        for Farm in ['Farmed','Wild']:
            for Trophic in data_tl.groups.keys():
                select_data = data[(data['Alpha3']==country.alpha3) &                                    (data['Freshwater/Seawater']==Water) &                                    (data['Farmed/Wild']==Farm) &                                    (data['Trophic Category']==Trophic)]
                if len(select_data) > 0:
                    Mean_Hg,GMean_Hg,Std_Hg,GStd_Hg = weighted_avg_and_std(select_data['Hg Level'],select_data['Sample Size'])
                    Flag = 'Local'
                else:
                    if (Trophic,Water,Farm) in Means_Hg.keys():
                        Mean_Hg = Means_Hg[Trophic,Water,Farm]
                        GMean_Hg = GMeans_Hg[Trophic,Water,Farm]
                        Std_Hg = Stds_Hg[Trophic,Water,Farm]
                        GStd_Hg = GStds_Hg[Trophic,Water,Farm]
                        Flag = 'Global'
                    else:
                        Mean_Hg = math.nan
                        GMean_Hg = math.nan
                        Std_Hg = math.nan
                        GStd_Hg = math.nan
                        Flag = 'Missing'
                country_database=([country.country,country.alpha3,Water,Farm,Trophic,GMean_Hg,GStd_Hg,Flag])
                countries_database.loc[len(countries_database.index)+1] = country_database


# In[24]:


countries_database.dropna(how='all').drop_duplicates().to_excel('Fish-MeHg-Country.xlsx',index=False)


# In[25]:


# put data into a database, don't specify countries
countries_database = pd.DataFrame(columns=['country','alpha3','water','farm','trophic','Hg','std','flag'])   
for country in country_name.iloc:
    for Water in ['Seawater','Freshwater']:
        for Farm in ['Farmed','Wild']:
            for Trophic in data_tl.groups.keys():
                if (Trophic,Water,Farm) in Means_Hg.keys():
                    Mean_Hg = Means_Hg[Trophic,Water,Farm]
                    GMean_Hg = GMeans_Hg[Trophic,Water,Farm]
                    GEMean_Hg = GEMeans_Hg[Trophic,Water,Farm]
                    Std_Hg = Stds_Hg[Trophic,Water,Farm]
                    GStd_Hg = GStds_Hg[Trophic,Water,Farm]
                    GEStd_Hg = GEStds_Hg[Trophic,Water,Farm]
                    Flag = 'Global'
                else:
                    Mean_Hg = math.nan
                    GMean_Hg = math.nan
                    GEMean_Hg = math.nan
                    Std_Hg = math.nan
                    GStd_Hg = math.nan
                    GEStd_Hg = math.nan
                    Flag = 'Missing'
                country_database=([country.country,country.alpha3,Water,Farm,Trophic,GEMean_Hg,GEStd_Hg,Flag])
                countries_database.loc[len(countries_database.index)+1] = country_database


# In[26]:


countries_database.dropna(how='all').drop_duplicates().to_excel('Fish-MeHg-Global.xlsx',index=False)


# In[27]:


# function to draw geodata
def draw_colorpleth(select_data,title):
    fig = go.Figure(data=go.Choropleth(                     locations=select_data['alpha3'],                     z=select_data['Hg'],                     #locationmode=select_data['country'],  \
                    colorscale='Reds', \
                    colorbar_title = 'Fish Hg, ppm<br>' + title))
    fig.show()


# In[28]:


# draw maps
for Water in ['Seawater','Freshwater']:
    for Farm in ['Farmed','Wild']:
        for Trophic in data_tl.groups.keys():
            select_data = countries_database[(countries_database['water']==Water) &                             (countries_database['farm']==Farm) &                             (countries_database['trophic']==Trophic)]
            title = Water + '<br>' + Farm + '<br>' + str(Trophic)
            draw_colorpleth(select_data,title)


# In[ ]:




