#!/usr/bin/env python
# coding: utf-8

# In[1]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import pycountry_convert as pc


# In[2]:


# load data
# local fish MeHg data is not helpful, EU actually has higher fish [MeHg]
# EU vs US: former has higher fish consumption, higher trophical level, higher [MeHg]. Overall 3-4 times higher of IQ loss
Exposure = pd.read_excel('../Exposure/Exposure(u+o)-freshlocal-seaglobal-MeHg-Country-ugd-1.xlsx') # ug MeHg per day
#Exposure = pd.read_excel('../Exposure/Exposure(u+o)-freshlocal-sealocal-MeHg-Country-ugd-1.xlsx') # ug MeHg per day
#Newborn
Newborn = pd.read_excel('risk_input_data.xlsx','Newborns')                # 1000 for 5 years, https://population.un.org/wpp/Graphs/DemographicProfiles/Line/900
# GDP
GDP = pd.read_excel('risk_input_data.xlsx','GDP')                        # k USD per capita, OECD SSP2
# HAI
HAIR = pd.read_excel('risk_input_data.xlsx','HAI')                        # x10-5
# Population
POP = pd.read_excel('risk_input_data.xlsx','Population')                  # 1000, https://population.un.org/wpp/Graphs/DemographicProfiles/Line/900
# GDP


# In[3]:


# Initial Risk
Risk = Exposure[Exposure.alpha3 != 'ATA']


# In[4]:


# lookup values of newborn, gdp, hai, and pop
Risk = pd.merge(Risk,Newborn[['alpha3','newborn_2010']],on='alpha3',how='left')
Risk = pd.merge(Risk,GDP[['alpha3','GDP_2010']],on='alpha3',how='left')
Risk = pd.merge(Risk,HAIR[['alpha3','IHD_Male','IHD_Female']],on='alpha3',how='left')
Risk = pd.merge(Risk,POP[['alpha3','Male_2010','Female_2010']],on='alpha3',how='left')


# In[5]:


# calculate total exposure, and fraction from each pathway
Risk['Exposure, ug/day'] = 10**Risk['freshfish_u'] + 10**Risk['seafish_u'] + 10**Risk['rice_u']
Risk['f_freshfish'] = 10**Risk['freshfish_u'] / Risk['Exposure, ug/day']
Risk['f_seafish'] = 10**Risk['seafish_u'] / Risk['Exposure, ug/day']
Risk['f_rice'] = 10**Risk['rice_u'] / Risk['Exposure, ug/day']


# In[6]:


# calculate IQ decrease
beta = 0.6    # ug Hg/L blood per ug Hg/day use 0.6+/-0.09
gamma = 0.3   # ug Hg/g hair 0.0135-0.567
lamda1 = 0.2  # ug Hg/g hair per ug Hg/L blood for pregnent women，0.2+/-0.0014
lamda2 = 0.18 # for other adults

Risk['IQ'] = Risk['Exposure, ug/day']*gamma*lamda1*beta


# In[7]:


# blood and hair Hg
Risk['blood Hg, ug/L'] = Risk['Exposure, ug/day'] * beta
Risk['hair Hg, ug/g'] = Risk['blood Hg, ug/L'] * lamda2


# use Bellanger 2013 function, it is lower for low exposure (e.g. CHN, USA), but similar for high exposure (e.g. EU)
# 0.2926 transfer mean to median hair Hg concentrations
# def f_dose_effect(row):
#     if row['hair Hg, ug/g'] > 0.4:
#         return 0.465*row['hair Hg, ug/g']-0.14095
#     elif row['hair Hg, ug/g'] <= 0.4:
#         return 0.1125*row['hair Hg, ug/g']
#     else:
#         return np.nan
#     
# Risk['IQ2'] = Risk.apply(lambda row:f_dose_effect(row),axis=1)

# In[8]:


# calculate TIQ, total IQ point = IQ x newborn, unit: points
Risk['TIQ'] = Risk['IQ'] * Risk['newborn_2010'] / 5.0 * 1000.0
#Risk['TIQ2'] = Risk['IQ2'] * Risk['newborn_2010'] / 5.0 * 1000.0


# In[9]:


# calculate MIQ, monetary IQ, USD
gdp_us_2010 = 42.164  #US GDP PPP per capita in 2010, unit of kilo 2005 dollar
USD2008_per_IQ = 18832
USD2008_to_USD2020 = 113.48/110.37*119.36/100.00 #2008->2010->2020

Risk['MIQ'] = Risk['TIQ']*Risk['GDP_2010']/gdp_us_2010*USD2008_per_IQ*USD2008_to_USD2020


# In[10]:


# Heart attack incidence
prob = 1.0/3.0 
f_fha_ihd= 0.5
phi = 0.066   # Risk per ug Hg/g hair
omega = 1.0    # dimensionless

Risk['HAI'] = prob*Risk['Male_2010']*1000.0*Risk['IHD_Male']*f_fha_ihd/100000.0*omega*              (1.0-np.exp(-phi*lamda2*beta*Risk['Exposure, ug/day'])) +                prob*Risk['Female_2010']*1000.0*Risk['IHD_Female']*f_fha_ihd/100000.0*omega*              (1.0-np.exp(-phi*lamda2*beta*Risk['Exposure, ug/day']))


# In[11]:


# VSL of HAI
USD2005_per_death = 6.3e6  # Giang PNAS 2016
USD2005_to_USD2020 = 113.48/100.0*119.36/100.00 #2005->2010->2020
Risk['HAM'] = Risk['HAI']*Risk['GDP_2010']/gdp_us_2010*USD2005_per_death*USD2005_to_USD2020 


# In[12]:


# Total Loss
Risk['Total Loss'] = Risk['MIQ'] + Risk['HAM']   # USD2010


# load marine landing trophic level index
# marine_landing_trophic = pd.read_excel('../Food Consumption/鱼类消费数据/Marine fishing landing trophic level.xlsx')
# Risk = pd.merge(Risk,marine_landing_trophic,on='alpha3',how='left')

# In[13]:


data = []
layout = dict()

key = 'geo1'  #IQ
data.append(dict(
                type='choropleth',
                locations=Risk['alpha3'], 
                z=Risk['IQ'], 
                zmax=0.3,
                zmin=0, 
                colorscale='hot_r', 
                colorbar_title = 'IQ',
                geo=key))
layout[key] = dict(domain = dict( x = [0.16,0.5], y = [0.77,1] ))

key = 'geo2'  #HAI
data.append(dict(
                type='choropleth',
                locations=Risk['alpha3'], 
                z=Risk['HAI'], 
                zmax=6000,
                zmin=0, 
                colorscale='hot_r', 
                colorbar_title = 'HAI',
                geo=key))
layout[key] = dict(domain = dict( x = [0.5,0.84], y = [0.77,1] ))

key = 'geo3'  #MIQ
data.append(dict(
                type='choropleth',
                locations=Risk['alpha3'], 
                z=np.log10(Risk['MIQ']), 
                zmax=10,
                zmin=6, 
                colorscale='hot_r', 
                colorbar_title = 'MIQ',
                geo=key))
layout[key] = dict(domain = dict( x = [0.16,0.5], y = [0.545,0.775] ))

key = 'geo4'  #HAM
data.append(dict(
                type='choropleth',
                locations=Risk['alpha3'], 
                z=np.log10(Risk['HAM']), 
                zmax=10,
                zmin=6, 
                colorscale='hot_r', 
                colorbar_title = 'HAM',
                geo=key))
layout[key] = dict(domain = dict( x = [0.5,0.84], y = [0.545,0.775] ))

key = 'geo5'  #Total Loss
data.append(dict(
                type='choropleth',
                locations=Risk['alpha3'], 
                z=np.log10(Risk['Total Loss']), 
                zmax=11,
                zmin=6, 
                colorscale='hot_r', 
                colorbar_title = 'Total Loss',
                geo=key))
layout[key] = dict(domain = dict( x = [0.16,0.84], y = [0.085,0.55] ))


fig = go.Figure(data=data,layout=layout)
fig.update_layout(width=2700,height=2025)
fig.show()
#fig.write_image("images/allpanels.png")


# In[14]:


# plot exposure for 3 pathways
fig = go.Figure(data=go.Choropleth(                     locations=Risk['alpha3'],                     z=10**Risk['freshfish_u'],                     zmax=5,
                    zmin=0, 
                    colorscale='Reds', \
                    colorbar_title = 'fresh fish'))
fig.show()
#fig.write_image("images/exposure_fwfish.png")

fig = go.Figure(data=go.Choropleth(                     locations=Risk['alpha3'],                     z=10**Risk['seafish_u'],                     zmax=10,
                    zmin=0,                                    
                    colorscale='Reds', \
                    colorbar_title = 'sea fish'))
fig.show()
#fig.write_image("images/exposure_seafood.png")

fig = go.Figure(data=go.Choropleth(                     locations=Risk['alpha3'],                     z=10**Risk['rice_u'],                     zmax=1.5,
                    zmin=0, 
                    colorscale='Reds', \
                    colorbar_title = 'rice'))
fig.show()
#fig.write_image("images/exposure_rice.png")


# In[15]:


# lookup individual contry
for index,risk in Risk.iterrows():
    if risk['alpha3'] == 'CHN':
        print(risk)


# In[16]:


# lookup individual contry
for index,risk in Risk.iterrows():
    if risk['alpha3'] == 'USA':
        print(risk)


# In[17]:


# global total
print(str(int(Risk['TIQ'].sum())) + ' total IQ points loss')
print(str(Risk['TIQ'].sum()/(Risk['newborn_2010'].sum()*1000/5)) + ' per-fetus IQ points loss')
print(str(int(Risk['MIQ'].sum()/1e9)) + ' Billion 2010USD total earn loss')
print(str(int(Risk['HAI'].sum())) + ' HAI death')
print(str(int(Risk['HAM'].sum()/1e9)) + ' Billion 2010USD HAI Value of statistical life')


# In[18]:


# get continent from alpha3
def get_continent(row):
    alpha3 = row['alpha3']
    if pd.isna(alpha3):
        return np.nan
    if alpha3 == 'BYS': # These countries cannot be handled by the pycountry_convert library
        continent = 'EU'
    elif alpha3 == 'ZAR':
        continent = 'AF'
    elif alpha3 == 'ATF':
        continent = 'AF'
    elif alpha3 == 'GBM':
        continent = 'EU'
    elif alpha3 == 'JTN':
        continent = 'NA'
    elif alpha3 == 'UNK':
        continent = 'EU'
    elif alpha3 == 'MID':
        continent = 'NA'   
    elif alpha3 == 'ANT':
        continent = 'SA'   
    elif alpha3 == 'MEX':  # Mexico belongs to NA geographically, but bw same as SA
        continent = 'SA'   
    elif alpha3 == 'PCN':
        continent = 'AS'     
    elif alpha3 == 'ESH':
        continent = 'AF' 
    elif alpha3 == 'WAK':
        continent = 'NA' 
    elif alpha3 == 'ATA':
        continent = 'AT'
    else:
        continent = pc.country_alpha2_to_continent_code(pc.country_alpha3_to_country_alpha2(alpha3))
    return continent

Risk['continent'] = Risk.apply(lambda row:get_continent(row), axis=1)


# In[19]:


# Calculate total MIQ for different continents
Risk_continent = Risk.groupby('continent')
Risk_continent.agg(sum)['Total Loss']/1e9


# In[20]:


# Compare with Bellanger list
Bellanger_list = ['AUT','BEL','BGR','DNK','IRL','EST','CZE','FIN','FRA',
                 'DEU','GRC','HRV','HUN','ITA','LVA','LTU','SVK','LUX',
                 'MLT','NLD','NOR','POL','PRT','ROU','SVN','ESP','SWE',
                 'CHE','GBR']
TIQ_LIST = 0
MIQ_LIST = 0
for index,risk in Risk.iterrows():
    if risk['alpha3'] in Bellanger_list:
        TIQ_LIST += risk['TIQ']
        MIQ_LIST += risk['MIQ']        
        #print([risk['alpha3'],risk['IQ'],int(risk['TIQ'])])
        #print(risk['alpha3'],risk['hair Hg, ug/g'])
        #print(risk['hair Hg, ug/g'])
        #print(risk['Marine trophic index'])



