#!/usr/bin/env python
# coding: utf-8

# In[3]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go


# In[4]:


# get country list
country_name=pd.read_excel('../food-hg/Shape_Country_Name.xlsx')


# In[5]:


# get database
data_ocean = pd.read_excel('1961-2013人口+海洋食物人均供给量/全球各国海洋产品.xlsx')
data_land = pd.read_excel('1961-2013人口+陆地食物人均供给量/全球各国陆地产品.xlsx')


# In[6]:


# only food consumtion
data_ocean = data_ocean[data_ocean['Element']=='Food supply quantity (kg/capita/yr)']
data_land = data_land[data_land['Element']=='Food supply quantity (kg/capita/yr)']


# In[7]:


# only 2009-2013
data_ocean = data_ocean[data_ocean['Year'] >= 2009]
data_land = data_land[data_land['Year'] >= 2009]


# In[8]:


# put data into a database
# function to sum all the positive terms
def posmean(datain):
    posdata = datain[datain>0]
    if len(posdata)>0:
        result = np.mean(posdata)
    else:
        result = 0.0
    return result

# function to calculate sum of terms
def nansum2(datain):
    if np.all(np.isnan(datain)):
        return np.nan
    else:
        return np.nansum(datain)

countries_database = pd.DataFrame(columns=['country','alpha3','Crustaceans', 'Demersal Fish', 'Fish, Body Oil',
                                           'Fish, Liver Oil', 'Marine Fish, Other','Pelagic Fish',
                                           'Freshwater Fish','Aquatic Animals, Others',
                                           'Rice (Milled Equivalent)','Total Marine Fish','Total Freshwater Fish'])   
for country in country_name.iloc:
    country_database=[country.country,country.alpha3]
    for Food in ['Crustaceans', 'Demersal Fish', 'Fish, Body Oil','Fish, Liver Oil', 'Marine Fish, Other','Pelagic Fish']:
        select_data = data_ocean[(data_ocean['Alpha3']==country.alpha3) & (data_ocean['Item']==Food)]
        if len(select_data) > 0:
            mean_food = posmean(select_data['Value'])
        else:
            mean_food = math.nan
        country_database.append(mean_food)
    for Food in ['Freshwater Fish','Aquatic Animals, Others','Rice (Milled Equivalent)']:
        select_data = data_land[(data_land['Alpha3']==country.alpha3) & (data_land['Item']==Food)]
        if len(select_data) > 0:
            mean_food = posmean(select_data['Value'])
        else:
            mean_food = math.nan            
        country_database.append(mean_food)
    country_database.append(nansum2(country_database[2:-3]))
    country_database.append(nansum2(country_database[-4:-2]))    
    countries_database.loc[len(countries_database.index)+1] = country_database


# In[9]:


#fill the gap with nearby region, access to coast, and income level
proxies = [['GRL', 'ISL'], ['UNK', 'MKD'], ['SYR', 'BGR'], ['LBY', 'EGY'], ['ERI', 'YEM'], ['SSD', 'CAF'],['QAT', 'KWT'],['SOM', 'YEM'], ['GNQ', 'GAB'], ['COD', 'COG'], ['BTN', 'NPL'], ['PNG', 'IDN'], ['ESH', 'MAR'], ['AFG', 'TJK']]

for proxy in proxies:
    ind_s = countries_database[countries_database['alpha3']==proxy[0]].index[0]
    ind_t = countries_database[countries_database['alpha3']==proxy[1]].index[0]

    if np.isnan(countries_database.loc[ind_s,'Total Marine Fish']):
        countries_database.loc[ind_s,['Crustaceans', 'Demersal Fish', 'Fish, Body Oil',                                       'Fish, Liver Oil', 'Marine Fish, Other','Pelagic Fish','Total Marine Fish']] =         countries_database.loc[ind_t,['Crustaceans', 'Demersal Fish', 'Fish, Body Oil',                                       'Fish, Liver Oil', 'Marine Fish, Other','Pelagic Fish','Total Marine Fish']]
    if np.isnan(countries_database.loc[ind_s,'Total Freshwater Fish']):
        countries_database.loc[ind_s,['Freshwater Fish','Aquatic Animals, Others','Total Freshwater Fish']] =         countries_database.loc[ind_t,['Freshwater Fish','Aquatic Animals, Others','Total Freshwater Fish']]
    if np.isnan(countries_database.loc[ind_s,'Rice (Milled Equivalent)']):
        countries_database.loc[ind_s,'Rice (Milled Equivalent)'] =         countries_database.loc[ind_t,'Rice (Milled Equivalent)']
    


# In[10]:


# drop detailed data
countries_database = countries_database[['country','alpha3','Rice (Milled Equivalent)','Total Marine Fish','Total Freshwater Fish']]


# In[11]:


# now seperate the farm/wild fractions
# get database
farm_ocean = pd.read_excel('鱼类消费数据/Freshwater Aquaculture.xlsx')
farm_land = pd.read_excel('鱼类消费数据/Marine Aquaculture.xlsx')


# In[12]:


countries_database = pd.merge(countries_database, farm_ocean, on='alpha3',how='outer')
countries_database = pd.merge(countries_database, farm_land, on='alpha3',how='outer')


# In[13]:


countries_database['f_farm_land'] = countries_database['farm_land']/countries_database['Total Freshwater Fish']
countries_database['f_farm_ocean'] = countries_database['farm_ocean']/countries_database['Total Marine Fish']


# In[14]:


ind = countries_database['f_farm_land']>1
countries_database.loc[ind,'f_farm_land']=0.8
ind = np.isnan(countries_database['f_farm_land'])
countries_database.loc[ind,'f_farm_land']=0

ind = countries_database['f_farm_ocean']>1
countries_database.loc[ind,'f_farm_ocean']=0.8
ind = np.isnan(countries_database['f_farm_ocean'])
countries_database.loc[ind,'f_farm_ocean']=0


# In[15]:


# get the marine fishing landing trophic level
marine_landing_trophic = pd.read_excel('鱼类消费数据/Marine fishing landing trophic level.xlsx')


# In[16]:


countries_database = pd.merge(countries_database, marine_landing_trophic, on='alpha3',how='outer')


# In[17]:


countries_database['Total Fish']=countries_database['Total Marine Fish']+countries_database['Total Freshwater Fish']
countries_database['f_ocean'] = countries_database['Total Marine Fish']/countries_database['Total Fish']
ind = np.isnan(countries_database['f_ocean'])
countries_database.loc[ind,'f_ocean']=0


# In[18]:


# drop detailed data
countries_database = countries_database[['country','alpha3','Rice (Milled Equivalent)','Total Fish',
                                         'f_ocean','f_farm_land','f_farm_ocean','Marine trophic index']]


# In[19]:


# assign default trophic structure
countries_database['f_Fresh_Farm_2'] = 0.511
countries_database['f_Fresh_Farm_3'] = 0.420
countries_database['f_Fresh_Farm_4'] = 0.069
countries_database['f_Fresh_Farm_5'] = 0

countries_database['f_Fresh_Wild_2'] = 0.477
countries_database['f_Fresh_Wild_3'] = 0.346
countries_database['f_Fresh_Wild_4'] = 0.176
countries_database['f_Fresh_Wild_5'] = 0.001

countries_database['f_Sea_Farm_2'] = 0.580
countries_database['f_Sea_Farm_3'] = 0.300
countries_database['f_Sea_Farm_4'] = 0.119
countries_database['f_Sea_Farm_5'] = 0.001

countries_database['f_Sea_Wild_2'] = 0.225
countries_database['f_Sea_Wild_3'] = 0.495
countries_database['f_Sea_Wild_4'] = 0.279
countries_database['f_Sea_Wild_5'] = 0.001


# #modify sea wild trophic level based on Marine trophic index
# #okay if linear, but not linear
# for index in countries_database.index:
#     if countries_database.loc[index,'Marine trophic index'] > 4:
#         countries_database.loc[index,'f_Sea_Wild_2'] = 0
#         countries_database.loc[index,'f_Sea_Wild_3'] = 0
#         countries_database.loc[index,'f_Sea_Wild_4'] = (0.75 - (countries_database.loc[index,'Marine trophic index'] - 4))/0.75
#         countries_database.loc[index,'f_Sea_Wild_5'] = (countries_database.loc[index,'Marine trophic index'] - 4)/0.75
#     elif countries_database.loc[index,'Marine trophic index'] > 3:
#         countries_database.loc[index,'f_Sea_Wild_2'] = 0
#         countries_database.loc[index,'f_Sea_Wild_3'] = 1 - (countries_database.loc[index,'Marine trophic index'] - 3)
#         countries_database.loc[index,'f_Sea_Wild_4'] = countries_database.loc[index,'Marine trophic index'] - 3
#         countries_database.loc[index,'f_Sea_Wild_5'] = 0
#     elif countries_database.loc[index,'Marine trophic index'] > 2:
#         countries_database.loc[index,'f_Sea_Wild_2'] = (0.75 - (countries_database.loc[index,'Marine trophic index'] - 2.25))/0.75
#         countries_database.loc[index,'f_Sea_Wild_3'] = (countries_database.loc[index,'Marine trophic index'] - 2.25)/0.75
#         countries_database.loc[index,'f_Sea_Wild_4'] = 0
#         countries_database.loc[index,'f_Sea_Wild_5'] = 0

# In[20]:


# calculate fish consumption
countries_database['Sea_Wild_2'] = countries_database['Total Fish'] * countries_database['f_ocean'] * (1-countries_database['f_farm_ocean']) * countries_database['f_Sea_Wild_2']
countries_database['Sea_Wild_3'] = countries_database['Total Fish'] * countries_database['f_ocean'] * (1-countries_database['f_farm_ocean']) * countries_database['f_Sea_Wild_3']
countries_database['Sea_Wild_4'] = countries_database['Total Fish'] * countries_database['f_ocean'] * (1-countries_database['f_farm_ocean']) * countries_database['f_Sea_Wild_4']
countries_database['Sea_Wild_5'] = countries_database['Total Fish'] * countries_database['f_ocean'] * (1-countries_database['f_farm_ocean']) * countries_database['f_Sea_Wild_5']

countries_database['Sea_Farm_2'] = countries_database['Total Fish'] * countries_database['f_ocean'] * countries_database['f_farm_ocean'] * countries_database['f_Sea_Farm_2']
countries_database['Sea_Farm_3'] = countries_database['Total Fish'] * countries_database['f_ocean'] * countries_database['f_farm_ocean'] * countries_database['f_Sea_Farm_3']
countries_database['Sea_Farm_4'] = countries_database['Total Fish'] * countries_database['f_ocean'] * countries_database['f_farm_ocean'] * countries_database['f_Sea_Farm_4']
countries_database['Sea_Farm_5'] = countries_database['Total Fish'] * countries_database['f_ocean'] * countries_database['f_farm_ocean'] * countries_database['f_Sea_Farm_5']

countries_database['Fresh_Wild_2'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * (1-countries_database['f_farm_land']) * countries_database['f_Fresh_Wild_2']
countries_database['Fresh_Wild_3'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * (1-countries_database['f_farm_land']) * countries_database['f_Fresh_Wild_3']
countries_database['Fresh_Wild_4'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * (1-countries_database['f_farm_land']) * countries_database['f_Fresh_Wild_4']
countries_database['Fresh_Wild_5'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * (1-countries_database['f_farm_land']) * countries_database['f_Fresh_Wild_5']

countries_database['Fresh_Farm_2'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * countries_database['f_farm_land'] * countries_database['f_Fresh_Farm_2']
countries_database['Fresh_Farm_3'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * countries_database['f_farm_land'] * countries_database['f_Fresh_Farm_3']
countries_database['Fresh_Farm_4'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * countries_database['f_farm_land'] * countries_database['f_Fresh_Farm_4']
countries_database['Fresh_Farm_5'] = countries_database['Total Fish'] * (1 - countries_database['f_ocean']) * countries_database['f_farm_land'] * countries_database['f_Fresh_Farm_5']


# In[21]:


# country-specific data, ignore them due to potential issues
L_apply_localdata = False
if L_apply_localdata:
    other_data = pd.read_excel('鱼类消费数据/Fish_Data.xlsx')

    # overwrite total fish consumption
    for index1 in other_data.index:
        index2 = countries_database[countries_database['alpha3'] == other_data.loc[index1,'alpha3']].index
        if np.size(index2) > 0:
            if other_data.loc[index1,'Total Fish'] > 0:
                countries_database.loc[index2[0],'Total Fish']=other_data.loc[index1,'Total Fish']
            
    # overwrite FAO data with country-specific data. Need to seperate the total fish concumption as some countries only have total fish
    for index1 in other_data.index:
        index2 = countries_database[countries_database['alpha3'] == other_data.loc[index1,'alpha3']].index
        if np.size(index2) > 0:
            if other_data.loc[index1,'Fresh_Farm_2']>=0:
                countries_database.loc[index2[0],['Fresh_Farm_2','Fresh_Farm_3','Fresh_Farm_4','Fresh_Farm_5']] =                 other_data.loc[index1,['Fresh_Farm_2','Fresh_Farm_3','Fresh_Farm_4','Fresh_Farm_5']]
                countries_database.loc[index2[0],['Fresh_Wild_2','Fresh_Wild_3','Fresh_Wild_4','Fresh_Wild_5']] =                 other_data.loc[index1,['Fresh_Wild_2','Fresh_Wild_3','Fresh_Wild_4','Fresh_Wild_5']]
                countries_database.loc[index2[0],['Sea_Farm_2','Sea_Farm_3','Sea_Farm_4','Sea_Farm_5']] =                 other_data.loc[index1,['Sea_Farm_2','Sea_Farm_3','Sea_Farm_4','Sea_Farm_5']]
                countries_database.loc[index2[0],['Sea_Wild_2','Sea_Wild_3','Sea_Wild_4','Sea_Wild_5']] =                 other_data.loc[index1,['Sea_Wild_2','Sea_Wild_3','Sea_Wild_4','Sea_Wild_5']]


# In[22]:


countries_database.dropna(how='all').drop_duplicates().to_excel('Fish-Consumption-Country.xlsx',index=False)


# In[24]:


# Map the distribution of fish consumption
fig = go.Figure(data=go.Choropleth(                     locations=countries_database['alpha3'],                     z=countries_database['Total Fish']*(1-countries_database['f_ocean']),                     text=countries_database['country'],                     locationmode='ISO-3',                     colorscale='Reds',                     colorbar_title = 'FW Fish<br>kg/capita/yr'))
fig.show()


# In[ ]:




