#!/usr/bin/env python
# coding: utf-8

# In[1]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go
from plotly.subplots import make_subplots
#import pycountry_convert as pc


# In[2]:


# load data
# local fish MeHg data is not helpful, EU actually has higher fish [MeHg]
# EU vs US: former has higher fish consumption, higher trophical level, higher [MeHg]. Overall 3-4 times higher of IQ loss
Exposure = pd.read_excel('../Exposure/Exposure(u+o)-freshlocal-seaglobal-MeHg-Country-ugd-1.xlsx') # ug MeHg per day
#Exposure = pd.read_excel('../Exposure/Exposure(u+o)-freshlocal-sealocal-MeHg-Country-ugd-1.xlsx') # ug MeHg per day
#Newborn
Newborn = pd.read_excel('../risk/risk_input_data.xlsx','Newborns')                # 1000 for 5 years
# GDP
GDP = pd.read_excel('../risk/risk_input_data.xlsx','GDP')                        # k USD per capita, OECD SSP2
# HAI
HAIR = pd.read_excel('../risk/risk_input_data.xlsx','HAI')                        # x10-5
# Population
POP = pd.read_excel('../risk/risk_input_data.xlsx','Population')                  # 1000
# Scales soil and deposition
Scales0 = pd.read_excel('data/scale0.xlsx')
Scales1 = pd.read_excel('data/scale1.xlsx')
Scales2 = pd.read_excel('data/scale2.xlsx')
Scales3 = pd.read_excel('data/scale3.xlsx')
Scales4 = pd.read_excel('data/scale4.xlsx')
Scales = [Scales0,Scales1,Scales2,Scales3,Scales4]

# scales ocean
Oceans = np.loadtxt('./data/ocean_mehg_fcr.txt')


# In[3]:


# constants for risk calculation
beta = 0.6    # ug Hg/L blood per ug Hg/day use 0.6+/-0.09
gamma = 0.3   # ug Hg/g hair 0.0135-0.567
lamda1 = 0.2  # ug Hg/g hair per ug Hg/L blood for pregnent women，0.2+/-0.0014
lamda2 = 0.18 # for other adults

# USD
gdp_us_2010 = 42.164  #US GDP PPP per capita in 2010, unit of kilo 2005 dollar
USD2008_per_IQ = 18832
USD2008_to_USD2020 = 113.48/110.37*119.36/100.00 #2008->2010->2020

# Heart attack incidence
prob = 1.0/3.0 
f_fha_ihd= 0.5
phi = 0.066   # Risk per ug Hg/g hair
omega = 1.0    # dimensionless

# VSL of HAI
USD2005_per_death = 6.3e6  # Giang PNAS 2016
USD2005_to_USD2020 = 113.48/100.0*119.36/100.00 #2005->2010->2020


# In[4]:


# initial global risk
Future_risk = np.zeros((9,5,5))


# In[5]:


# Calculate risk for every 5 years
for iyear,year in enumerate(range(2010,2055,5)):
    years = str(year)
    for scenario,scale in enumerate(Scales):
        Risk = Exposure[Exposure.alpha3 != 'ATA'].copy()
        Risk = pd.merge(Risk,Newborn[['alpha3','newborn_'+years]],on='alpha3',how='left')
        Risk = pd.merge(Risk,GDP[['alpha3','GDP_'+years]],on='alpha3',how='left')
        Risk = pd.merge(Risk,HAIR[['alpha3','IHD_Male','IHD_Female']],on='alpha3',how='left')
        Risk = pd.merge(Risk,POP[['alpha3','Male_'+years,'Female_'+years]],on='alpha3',how='left')
        Risk = pd.merge(Risk,scale[['alpha3','soil2010','depo2010']],on='alpha3',how='left')
        if year != 2010:
            Risk = pd.merge(Risk,scale[['alpha3','soil'+years,'depo'+years]],on='alpha3',how='left')
        
        # calculate total exposure, and fraction from each pathway
        Risk['Exposure, ug/day'] = 10**Risk['freshfish_u']*Risk['depo'+years]/Risk['depo2010'] +                                    10**Risk['seafish_u']*Oceans[(year-2010)//5,scenario]/Oceans[0,scenario] +                                    10**Risk['rice_u']*Risk['soil'+years]/Risk['soil2010']
        Risk['f_freshfish'] = 10**Risk['freshfish_u']*Risk['depo'+years]/Risk['depo2010'] / Risk['Exposure, ug/day']
        Risk['f_seafish'] = 10**Risk['seafish_u']*Oceans[(year-2010)//5,scenario]/Oceans[0,scenario] / Risk['Exposure, ug/day']
        Risk['f_rice'] = 10**Risk['rice_u']*Risk['soil'+years]/Risk['soil2010'] / Risk['Exposure, ug/day']
        
        
        # calculate IQ decrease
        Risk['IQ'] = Risk['Exposure, ug/day']*gamma*lamda1*beta
        
        # blood and hair Hg
        Risk['blood Hg, ug/L'] = Risk['Exposure, ug/day'] * beta
        Risk['hair Hg, ug/g'] = Risk['blood Hg, ug/L'] * lamda2
        
        # calculate TIQ, total IQ point = IQ x newborn, unit: points
        Risk['TIQ'] = Risk['IQ'] * Risk['newborn_'+years] / 5.0 * 1000.0
        
        # calculate MIQ, monetary IQ, USD
        Risk['MIQ'] = Risk['TIQ']*Risk['GDP_'+years]/gdp_us_2010*USD2008_per_IQ*USD2008_to_USD2020
        
        # heart attack death incidence
        Risk['HAI'] = prob*Risk['Male_'+years]*1000.0*Risk['IHD_Male']*f_fha_ihd/100000.0*omega*              (1.0-np.exp(-phi*lamda2*beta*Risk['Exposure, ug/day'])) +                prob*Risk['Female_'+years]*1000.0*Risk['IHD_Female']*f_fha_ihd/100000.0*omega*              (1.0-np.exp(-phi*lamda2*beta*Risk['Exposure, ug/day']))
        Risk['HAM'] = Risk['HAI']*Risk['GDP_'+years]/gdp_us_2010*USD2005_per_death*USD2005_to_USD2020 
        
        # Total Loss
        Risk['Total Loss'] = Risk['MIQ'] + Risk['HAM']   # USD2020
        
        # put into large array
        Future_risk[iyear,:,scenario] = [Risk['TIQ'].sum(),Risk['MIQ'].sum(),Risk['HAI'].sum(),Risk['HAM'].sum(),Risk['Total Loss'].sum()]


# In[6]:


# billion usd in 2020
Future_risk*=1e-9


# In[7]:


# make plots
time_stamps = np.arange(2010,2055,5)
width = 0.8
offset = 1

p1 = plt.bar(time_stamps,Future_risk[:,1,0], width, color='b' )
p2 = plt.bar(time_stamps,Future_risk[:,3,0], width, bottom = Future_risk[:,1,0], color='r' )

p1 = plt.bar(time_stamps+offset,Future_risk[:,1,1], width, color='b' )
p2 = plt.bar(time_stamps+offset,Future_risk[:,3,1], width, bottom = Future_risk[:,1,1], color='r' )

p1 = plt.bar(time_stamps+2*offset,Future_risk[:,1,2], width, color='b' )
p2 = plt.bar(time_stamps+2*offset,Future_risk[:,3,2], width, bottom = Future_risk[:,1,2], color='r' )

p1 = plt.bar(time_stamps-offset,Future_risk[:,1,3], width, color='b' )
p2 = plt.bar(time_stamps-offset,Future_risk[:,3,3], width, bottom = Future_risk[:,1,3], color='r' )

p1 = plt.bar(time_stamps-2*offset,Future_risk[:,1,4], width, color='b' )
p2 = plt.bar(time_stamps-2*offset,Future_risk[:,3,4], width, bottom = Future_risk[:,1,4], color='r' )

plt.xlabel('Time, year')
plt.ylabel('Billion USD in 2020')
plt.show()


# In[8]:


colors = ['k','g','lightgreen','r','lightsalmon']

fig, ax = plt.subplots(3,2, sharey=False,clear=True,figsize=(8,10))

for scenario in range(5):
    ax[0,0].plot(time_stamps,1e9*Future_risk[:,0,scenario], colors[scenario])
ax[0,0].set_xlabel('Time, year')
ax[0,0].set_ylabel('TIQ, points per year')

for scenario in range(5):
    ax[0,1].plot(time_stamps,Future_risk[:,1,scenario], colors[scenario])
ax[0,1].set_xlabel('Time, year')
ax[0,1].set_ylabel('MIQ, billion USD2020 per year')

for scenario in range(5):
    ax[1,0].plot(time_stamps,1e9*Future_risk[:,2,scenario], colors[scenario])
ax[1,0].set_xlabel('Time, year')
ax[1,0].set_ylabel('HAI, deaths per year')

for scenario in range(5):
    ax[1,1].plot(time_stamps,Future_risk[:,3,scenario], colors[scenario])
ax[1,1].set_xlabel('Time, year')
ax[1,1].set_ylabel('HAM, billion USD2020 per year')

for scenario in range(5):
    ax[2,0].plot(time_stamps,Future_risk[:,4,scenario], colors[scenario])
ax[2,0].set_xlabel('Time, year')
ax[2,0].set_ylabel('Total Loss, billion USD2020 per year')

for scenario in range(5):
    ax[2,1].plot(time_stamps,Future_risk[:,4,scenario], colors[scenario])
    ax[2,1].plot(time_stamps,Future_risk[:,1,scenario], colors[scenario],dashes=[6,2])
    
ax[2,1].set_xlabel('Time, year')
ax[2,1].set_ylabel('Total Loss, billion USD2020 per year')


# In[9]:


colors = ['k','g','lightgreen','r','lightsalmon']
labels = ['CP','MFR','NP-Delayed','A1B','A2']

fig, ax = plt.subplots(1,3, sharey=False,clear=True,figsize=(15,4))

for scenario in range(5):
    ax[0].plot(time_stamps,1e3*Future_risk[:,0,scenario], colors[scenario],label=labels[scenario],marker='o')
ax[0].set_xlabel('Time, year')
ax[0].set_ylabel('IQ Decrease, $x10^6$ pts/y')
ax[0].legend()

for scenario in range(5):
    ax[1].plot(time_stamps,1e6*Future_risk[:,2,scenario], colors[scenario],marker='o')
ax[1].set_xlabel('Time, year')
ax[1].set_ylabel('Deaths, x1000/y')

for scenario in range(4,-1,-1):
    line1,=ax[2].plot(time_stamps,Future_risk[:,4,scenario], colors[scenario],marker='o')
    line2,=ax[2].plot(time_stamps,Future_risk[:,1,scenario], colors[scenario],dashes=[6,2],marker='o')
    
ax[2].set_xlabel('Time, year')
ax[2].set_ylabel('Economic Loss, $x10^9$ USD/y')
ax[2].legend((line1, line2), ('IQ Decrease', 'Total Loss'))

plt.savefig("images/global_risk_trends.jpg",dpi=300, facecolor='w', edgecolor='w')


# #Calculate benefits
# colors = ['b','g','lightgreen','r','lightsalmon']
# 
# fig, ax = plt.subplots(3,2, sharey=False,clear=True,figsize=(15,15))
# 
# for scenario in range(1,5):
#     ax[0,0].plot(time_stamps,1e9*(Future_risk[:,0,scenario]-Future_risk[:,0,0]), colors[scenario])
# ax[0,0].set_xlabel('Time, year')
# ax[0,0].set_ylabel('Changes in TIQ, points per year')
# 
# for scenario in range(1,5):
#     ax[0,1].plot(time_stamps,Future_risk[:,1,scenario]-Future_risk[:,1,0], colors[scenario])
# ax[0,1].set_xlabel('Time, year')
# ax[0,1].set_ylabel('Changes in MIQ, billion USD2020 per year')
# 
# for scenario in range(1,5):
#     ax[1,0].plot(time_stamps,1e9*(Future_risk[:,2,scenario]-Future_risk[:,2,0]), colors[scenario])
# ax[1,0].set_xlabel('Time, year')
# ax[1,0].set_ylabel('Changes in HAI, deaths per year')
# 
# for scenario in range(1,5):
#     ax[1,1].plot(time_stamps,Future_risk[:,3,scenario]-Future_risk[:,3,0], colors[scenario])
# ax[1,1].set_xlabel('Time, year')
# ax[1,1].set_ylabel('Changes in HAM, billion USD2020 per year')
# 
# for scenario in range(1,5):
#     ax[2,0].plot(time_stamps,Future_risk[:,4,scenario]-Future_risk[:,4,0], colors[scenario])
# ax[2,0].set_xlabel('Time, year')
# ax[2,0].set_ylabel('Changes in Total Loss, billion USD2020 per year')

# In[10]:


# calculate cumulative economic loss, discount by 3% to 2050


# In[11]:


# discount factor
discount_rate = 0.03
discount_rates = (1+discount_rate)**np.arange(40,-5,-5)


# In[28]:


year=2050
years = str(year)
scale = Scales[1]

Risk = Exposure[Exposure.alpha3 != 'ATA'].copy()
Risk = pd.merge(Risk,Newborn[['alpha3','newborn_'+years]],on='alpha3',how='left')
Risk = pd.merge(Risk,GDP[['alpha3','GDP_'+years]],on='alpha3',how='left')
Risk = pd.merge(Risk,HAIR[['alpha3','IHD_Male','IHD_Female']],on='alpha3',how='left')
Risk = pd.merge(Risk,POP[['alpha3','Male_'+years,'Female_'+years]],on='alpha3',how='left')
Risk = pd.merge(Risk,scale[['alpha3','soil2010','depo2010']],on='alpha3',how='left')

if year != 2010:
    Risk = pd.merge(Risk,scale[['alpha3','soil'+years,'depo'+years]],on='alpha3',how='left')
        
        # calculate total exposure, and fraction from each pathway        
Risk['Exposure, ug/day'] = 10**Risk['freshfish_u']*Risk['depo'+years]/Risk['depo2010'] +                                    10**Risk['seafish_u']*Oceans[(year-2010)//5,scenario]/Oceans[0,scenario] +                                    10**Risk['rice_u']*Risk['soil'+years]/Risk['soil2010']
Risk['f_freshfish'] = 10**Risk['freshfish_u']*Risk['depo'+years]/Risk['depo2010'] / Risk['Exposure, ug/day']
Risk['f_seafish'] = 10**Risk['seafish_u']*Oceans[(year-2010)//5,scenario]/Oceans[0,scenario] / Risk['Exposure, ug/day']
Risk['f_rice'] = 10**Risk['rice_u']*Risk['soil'+years]/Risk['soil2010'] / Risk['Exposure, ug/day']
