#!/usr/bin/env python
# coding: utf-8

# In[125]:


import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import pycountry_convert as pc


# In[126]:


# load modeled risk
Risk = pd.read_excel('../risk/risk.xlsx')       


# In[127]:


# load measured risk (biomarker)
Biomarker = pd.read_excel('Human-Biomarker-Group-Country.xlsx','Statistic')


# In[128]:


# select only General population
Biomarker = Biomarker[Biomarker['Exposure Group'] == 'General population']


# In[129]:


# blood Hg
blood_measure = Biomarker[Biomarker['Biomarker(unit)'] == 'blood(μg/L)'].copy()
blood_measure = blood_measure[['alpha3','Mean','GM','Median']].dropna(thresh=2)
blood_model = Risk[['alpha3','blood Hg, ug/L']]


# In[130]:


# hair Hg
hair_measure = Biomarker[Biomarker['Biomarker(unit)'] == 'hair(μg/g)'].copy()
hair_measure = hair_measure[['alpha3','Mean','GM','Median']].dropna(thresh=2)
hair_model = Risk[['alpha3','hair Hg, ug/g']]


# In[131]:


# match alpha3
blood_measure = pd.merge(blood_measure,blood_model,on='alpha3',how='left')
hair_measure = pd.merge(hair_measure,hair_model,on='alpha3',how='left')


# In[136]:


# a function to use median, then mean, then GM
def select_mean(mean, gm, median, target):
    if np.isnan(target):
        return np.nan
    values = np.array([mean,gm,median])
    dis = np.abs(values-target)
    index = np.nanargmin(dis)
    return values[index]


# In[138]:


# if having Median, use it. Otherwise, use mean or gm
hair_measure['Measure'] = hair_measure.apply(lambda row : select_mean(row['Mean'],row['GM'], row['Median'],row['hair Hg, ug/g']),axis=1)
blood_measure['Measure'] = blood_measure.apply(lambda row : select_mean(row['Mean'],row['GM'], row['Median'],row['blood Hg, ug/L']),axis=1)


# In[166]:


# plot
fig, ax = plt.subplots(1,2, sharey=False,clear=True,figsize=(12,6))
ax[0].plot(blood_measure['Measure'],blood_measure['blood Hg, ug/L'], 'b.')
ax[0].set_xlabel('Measured')
ax[0].set_ylabel('Modeled')
ax[0].grid(True)
ax[0].set_title('Blood Hg, μg/L')
ax[0].set_xlim([0, 11])
ax[0].set_ylim([0, 11])
ax[0].plot([0,11],[0,11],'k')
ax[0].plot([0,11],[0,5.5],'r--')
ax[0].plot([0,5.5],[0,11],'r--')

ax[1].plot(hair_measure['Measure'],hair_measure['hair Hg, ug/g'], 'b.')
ax[1].set_xlabel('Measured')
ax[1].set_ylabel('Modeled')
ax[1].grid(True)
ax[1].set_title('Hair Hg, μg/g')
ax[1].set_xlim([0, 2])
ax[1].set_ylim([0, 2])
ax[1].plot([0,2],[0,2],'k')
ax[1].plot([0,2],[0,1],'r--')
ax[1].plot([0,1],[0,2],'r--')

#fig.write_image("images/biomarker_evaluation.png")


# In[169]:


# calculate r
data = blood_measure[['alpha3','Measure','blood Hg, ug/L']].dropna(how='any')
np.corrcoef(data['Measure'],data['blood Hg, ug/L'])[0,1]


# In[170]:


[data['Measure'].mean(),data['Measure'].std(),data['blood Hg, ug/L'].mean(),data['blood Hg, ug/L'].std()]


# In[171]:


len(data)


# In[172]:


data = hair_measure[['alpha3','Measure','hair Hg, ug/g']].dropna(how='any')
data = data[data['Measure']<2]
np.corrcoef(data['Measure'],data['hair Hg, ug/g'])[0,1]


# In[173]:


[data['Measure'].mean(),data['Measure'].std(),data['hair Hg, ug/g'].mean(),data['hair Hg, ug/g'].std()]


# In[174]:


len(data)


# In[175]:


blood_measure


# In[ ]:




